#!/usr/bin/env python
import os
from .exceptions import FileParsingException, FilesNotFoundException
from .parsers import FILE_EXTENSION_TO_PARSERS


def merge_configuration_from_dict_root(base_config, overrides):
    for k, v in overrides.items():
        if type(v) is dict and type(base_config.get(k)) is dict:
            merge_configuration_from_dict_root(base_config[k], v)
        else:
            base_config[k] = v


def get_parser_for_file(filename):
    _, extension = os.path.splitext(filename)
    return FILE_EXTENSION_TO_PARSERS[extension]


def parse_file(filename):
    if not os.path.isfile(filename):
        raise FilesNotFoundException(f"File {filename} doesn't exist!")

    parser = get_parser_for_file(filename)

    try:
        return parser.parse(filename)
    except Exception as e:
        raise FileParsingException(filename) from e



#!/usr/bin/env python
import itertools
import json
import os

from yaml import YAMLObject, Loader, load
from yaml.constructor import BaseConstructor
from yaml.nodes import ScalarNode, SequenceNode, MappingNode

from .exceptions import TagConstructionException, UnsupportedNodeTypeException


# monkey patch construct_object to ensure alias expansion occurs before our custom yaml tags refer to any aliases
def construct_object_deep(self, node, deep=True):
    return construct_object_orig(self, node, deep)


construct_object_orig = BaseConstructor.construct_object
BaseConstructor.construct_object = construct_object_deep


class ContextPassingYamlLoader(Loader):
    def __init__(self, stream, context):
        super().__init__(stream)
        self.context = context


class ArgListAcceptingYamlTag(YAMLObject):
    supported_node_types = (ScalarNode, SequenceNode, MappingNode)

    @classmethod
    def __get_handler_for_node_type(cls, parsing_node_type):
        handler_map = {
            node_type: handler for node_type, handler in [
                (ScalarNode, cls.map_scalar_node),
                (SequenceNode, cls.map_sequence_node),
                (MappingNode, cls.map_mapping_node),
            ] if node_type in cls.supported_node_types
        }

        return handler_map.get(parsing_node_type)

    @classmethod
    def __get_exception(cls, message, filename):
        return TagConstructionException(
            tag_name=cls.yaml_tag,
            filename=filename,
            message=message,
        )

    @classmethod
    def map_scalar_node(cls, loader, node):
        loaded_node = loader.construct_scalar(node)
        return cls.map_node_data(loader.context, loaded_node)

    @classmethod
    def map_sequence_node(cls, loader, node):
        loaded_node = loader.construct_sequence(node, deep=True)
        return cls.map_node_data(loader.context, *loaded_node)

    @classmethod
    def map_mapping_node(cls, loader, node):
        loaded_node = loader.construct_mapping(node, deep=True)
        return cls.map_node_data(loader.context, **loaded_node)

    @classmethod
    def map_node_data(cls, context, *args, **kwargs):
        raise NotImplementedError()

    @classmethod
    def handle_tag_construction_error(cls, message, filename, exc=None):
        if exc is not None:
            raise cls.__get_exception(message, filename) from exc
        else:
            raise cls.__get_exception(message, filename)

    @classmethod
    def from_yaml(cls, loader, node):
        handler = cls.__get_handler_for_node_type(type(node))

        if handler is None:
            raise UnsupportedNodeTypeException(cls, type(node))

        return handler(loader, node)


class JoinFilePaths(ArgListAcceptingYamlTag):
    yaml_tag = "!JoinFilePaths"
    supported_node_types = (SequenceNode, MappingNode)

    @classmethod
    def map_node_data(cls, context, *args, **kwargs):
        file_paths = kwargs.get("paths") or args

        if len(file_paths) == 0:
            cls.handle_tag_construction_error(
                message="No paths provided, provide them either with a 'paths' mapping, or a list of paths",
                filename=context["filename"],
            )

        return os.path.join(*file_paths)


class EnvVar(ArgListAcceptingYamlTag):
    yaml_tag = "!EnvVar"
    supported_node_types = (ScalarNode, MappingNode)

    @classmethod
    def map_node_data(cls, context, name, default=None):
        if name not in os.environ and default is None:
            cls.handle_tag_construction_error(
                message="Environment Variable '{}' not set, and no default provided!".format(name),
                filename=context["filename"],
            )

        return os.environ.get(name, default)


class Chain(ArgListAcceptingYamlTag):
    yaml_tag = "!Chain"
    supported_node_types = (SequenceNode, MappingNode)

    @classmethod
    def map_node_data(cls, context, *args, **kwargs):
        lists = kwargs.get("lists") or args
        for l in lists:
            if type(l) is not list:
                raise TagConstructionException(cls.yaml_tag, context["filename"], "All elements of !Chain node must be lists")

        return list(itertools.chain.from_iterable(lists))


class RelativeFileIncludingYamlTag(ArgListAcceptingYamlTag):
    supported_node_types = (ScalarNode, MappingNode)

    @classmethod
    def handle_included_file(cls, context, file_handle):
        raise NotImplementedError()

    @classmethod
    def map_node_data(cls, context, filename):
        current_file_directory = os.path.dirname(context["filename"])
        full_file_path = os.path.join(current_file_directory, filename)

        try:
            with open(full_file_path) as file_handle:
                return cls.handle_included_file(context, file_handle)

        except IOError as e:
            cls.handle_tag_construction_error(
                message="Attempted to include relative file {}, which doesn't exist!".format(filename),
                filename=context["filename"],
                exc=e,
            )


class IncludeJson(RelativeFileIncludingYamlTag):
    yaml_tag = "!IncludeJson"

    @classmethod
    def handle_included_file(cls, context, file_handle):
        try:
            return json.load(file_handle)
        except ValueError as e:

            cls.handle_tag_construction_error(
                message="Failed to parse relative json file {}!".format(file_handle.name),
                filename=context["filename"],
                exc=e,
            )


class IncludeYaml(RelativeFileIncludingYamlTag):
    yaml_tag = "!IncludeYaml"

    @classmethod
    def handle_included_file(cls, context, file_handle):
        try:
            context_passing_loader = lambda stream: ContextPassingYamlLoader(stream, {"filename": file_handle.name})
            return load(file_handle, Loader=context_passing_loader)
        except ValueError as e:
            cls.handle_tag_construction_error(
                message="Failed to parse relative yaml file {}!".format(file_handle.name),
                filename=context["filename"],
                exc=e,
            )


class IncludeText(RelativeFileIncludingYamlTag):
    yaml_tag = "!IncludeText"

    @classmethod
    def handle_included_file(cls, context, file_handle):
        return file_handle.read().strip()



#!/usr/bin/env python

class FilesNotFoundException(Exception):
    def __init__(self, msg):
        super().__init__(msg)


class FileParsingException(Exception):
    def __init__(self, filename):
        super().__init__("Exception thrown while processing {}".format(filename))


class TagConstructionException(Exception):
    def __init__(self, tag_name, filename, message):
        super().__init__("Error constructing {tag} tag in file {filename}. {message}".format(
            tag=tag_name,
            filename=filename,
            message=message,
        ))


class UnsupportedNodeTypeException(Exception):
    def __init__(self, tag_parser_type, node_type):
        super().__init__("Yaml Tag Parser {} cannot parse nodes of type {}!".format(tag_parser_type, node_type))



#!/usr/bin/env python
import json
import logging
import logging.config
import os

from .exceptions import FilesNotFoundException, FileParsingException
from .parsers import SUPPORTED_FILE_EXTENSIONS, CONFIG_FILENAME_FORMAT
from .utils import merge_configuration_from_dict_root, parse_file

_LOGGER = logging.getLogger(__name__)


def _get_configuration_dirs(configuration_dirs_arg):
    if configuration_dirs_arg is None:
        return (
            os.environ.get("JCONFIGURE_CONFIG_DIRECTORIES").split(",")
            if "JCONFIGURE_CONFIG_DIRECTORIES" in os.environ
            else [os.path.join(os.getcwd(), "config")]
        )

    elif type(configuration_dirs_arg) is basestring:
        return [configuration_dirs_arg]

    return configuration_dirs_arg


def _parse_file_handle_exceptions(filename, fail_on_parse_error):
    try:
        return parse_file(filename)
    except Exception as e:
        if fail_on_parse_error:
            _LOGGER.error("Exception thrown while parsing file {}!".format(filename))
            raise FileParsingException(filename) from e
        else:
            _LOGGER.warn("Exception thrown while parsing file {}. fail_on_parse_error is not set, continuing".format(
                filename
            ))

            return {}


def _merge_configuration_from_file(base_config, filename, fail_on_parse_error):
    overrides = _parse_file_handle_exceptions(filename, fail_on_parse_error)
    merge_configuration_from_dict_root(base_config, overrides)


def _find_available_config_files_in_directory(directory, basename):
    possible_config_files = (
        os.path.join(directory, CONFIG_FILENAME_FORMAT.format(basename=basename, extension=extension))
        for extension in SUPPORTED_FILE_EXTENSIONS
    )

    return [config_file for config_file in possible_config_files if os.path.isfile(config_file)]


def _handle_available_files_in_directories(
    base_config,
    file_basenames,
    configuration_dirs,
    fail_on_parse_error,
    fail_on_missing_files,
):
    basename_found = {b: False for b in file_basenames}

    for basename in file_basenames:
        for directory in configuration_dirs:
            config_files_in_dir = _find_available_config_files_in_directory(directory, basename)

            if len(config_files_in_dir) == 0:
                _LOGGER.debug("No config files for basename {} found in directory {}".format(basename, directory))
            else:
                basename_found[basename] = True

            for f in config_files_in_dir:
                _LOGGER.debug("Parsing file {} and merging with config".format(f))
                _merge_configuration_from_file(
                    base_config=base_config,
                    filename=f,
                    fail_on_parse_error=fail_on_parse_error,
                )

    for basename, found in basename_found.items():
        if fail_on_missing_files and not found:
            _LOGGER.error("No files found for basename {} in any directory and fail_on_missing_files is set, exiting".format(basename))
            raise FilesNotFoundException("No files found for basename {} in any directory".format(basename))


def _handle_available_defaults_files(
    base_config,
    defaults_basename,
    configuration_dirs,
    fail_on_parse_error,
    fail_on_missing_files,
):
    _LOGGER.debug("Searching for defaults config files...")
    _handle_available_files_in_directories(
        base_config=base_config,
        file_basenames=[defaults_basename],
        configuration_dirs=configuration_dirs,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
    )


def _handle_active_profiles_files(
    base_config,
    active_profiles,
    configuration_dirs,
    fail_on_parse_error,
    fail_on_missing_files,
):
    _LOGGER.debug("Searching for active profile config files...")
    _handle_available_files_in_directories(
        base_config=base_config,
        file_basenames=active_profiles,
        configuration_dirs=configuration_dirs,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
    )


def _configure_logging(
    configuration_dirs,
    logging_config_filename,
    fail_on_parse_error,
    fail_on_missing_files,
    overrides={},
):
    logging_config = {}
    _handle_available_files_in_directories(
        base_config=logging_config,
        file_basenames=[logging_config_filename],
        configuration_dirs=configuration_dirs,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
    )

    merge_configuration_from_dict_root(logging_config, overrides)
    logging.config.dictConfig(logging_config)
    _LOGGER.debug(f"Configured logging with config: {json.dumps(logging_config)}")


def configure(
    configuration_dirs=None,
    logging_config_filename="logging",
    defaults_basename="defaults",
    active_profiles=None,
    fail_on_parse_error=True,
    fail_on_missing_files=False,
    overrides={},
    logging_overrides={},
):
    """
    :param configuration_dirs: The directories from which configuration files will be pulled, either a single string, or
                               a list of strings. If None is passed, reads a comma seperated list of directories from
                               an environment variable JCONFIGURE_CONFIG_DIRECTORIES, if this variable is not set
                               defaults to a subdirectory of the current working directory called "config"
    :param logging_config_filename: If provided (not None), read config files matching this base name and any supported
                                    extension and use the read config to configure the _LOGGER
    :param defaults_basename: The name of the default config files that are read first, same as the profile, it is the
                              basename of the file to look for without an extension. Defaults to "defaults"
    :param active_profiles: The list of profiles currently active, if None is passed, this will be read from an
                            environment variable JCONFIGURE_ACTIVE_PROFILES. This variable should be a comma separated
                            list of strings. For each profile a file with the name {profile}.{extension} will be read,
                            where extension is one of the allowed file types
    :param fail_on_parse_error: If False suppress any exceptions thrown while processing a file. Defaults to True
    :param fail_on_missing_files: If True, raise an exception if an expected file is not found. Defaults to False
    :param overrides: If provided, will be deep merged with the configuration directory constructed
    :param logging_overrides: If provided, will be deep merged with the logging configuration before logging is configured

    :return: The configuration dictionary pulled from the configuration files specified under configuration_dir
    """
    configuration_dirs = _get_configuration_dirs(configuration_dirs)

    _configure_logging(
        configuration_dirs=configuration_dirs,
        logging_config_filename=logging_config_filename,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
        overrides=logging_overrides,
    )

    active_profiles = (
        active_profiles or
        (os.environ.get("JCONFIGURE_ACTIVE_PROFILES").split(",") if "JCONFIGURE_ACTIVE_PROFILES" in os.environ else [])
    )

    base_config = {}

    _LOGGER.info("Configuring Application using files in config directories [{}]".format(", ".join(configuration_dirs)))
    _LOGGER.info("Active profiles: [{}]".format(", ".join(active_profiles)))

    _handle_available_defaults_files(
        base_config=base_config,
        configuration_dirs=configuration_dirs,
        defaults_basename=defaults_basename,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
    )

    _handle_active_profiles_files(
        base_config=base_config,
        configuration_dirs=configuration_dirs,
        active_profiles=active_profiles,
        fail_on_parse_error=fail_on_parse_error,
        fail_on_missing_files=fail_on_missing_files,
    )

    merge_configuration_from_dict_root(base_config, overrides)
    _LOGGER.debug(f"Constructed config: {json.dumps(base_config)}")
    return base_config



#!/usr/bin/env python
import json
import os
import yaml

from .yaml_tags import *


class JsonConfigFileParser:
    FILE_EXTENSIONS = [".json"]

    @staticmethod
    def parse(filename):
        with open(filename) as json_file:
            return json.load(json_file)


class YamlConfigFileParser:
    FILE_EXTENSIONS = [".yaml", ".yml"]

    @staticmethod
    def parse(filename):
        with open(filename) as yaml_file:
            return yaml.load(
                yaml_file,
                Loader=lambda stream: ContextPassingYamlLoader(stream, {"filename": filename})
            )


AVAILABLE_FILE_PARSERS = [
    JsonConfigFileParser,
    YamlConfigFileParser,
]

FILE_EXTENSION_TO_PARSERS = {
    extension: parser
    for parser in AVAILABLE_FILE_PARSERS
    for extension in parser.FILE_EXTENSIONS
}

SUPPORTED_FILE_EXTENSIONS = list(FILE_EXTENSION_TO_PARSERS.keys())
CONFIG_FILENAME_FORMAT = "{basename}{extension}"



#!/usr/bin/env python
from distutils.core import setup

setup(
    name="jconfigure",
    packages=["jconfigure"],
    version="18.0515-dev4",
    description="A python configuration management library",
    author="John Norwood",
    author_email="norwood.john.m@gmail.com",
    url="https://github.com/norwoodj/jconfigure",
    download_url="https://github.com/norwoodj/jconfigure/archive/18.0515-dev.tar.gz",
    keywords=["configuration"],
    classifiers=[],
    install_requires=[
        "PyYAML==3.12",
    ]
)
